@@ -1,8 +1,10 @@
 import io
 import logging
+import multiprocessing as mp
 import time
 import typing as t
 from contextlib import contextmanager
+from functools import partial
 from pathlib import Path
 from types import TracebackType
 
@@ -309,6 +311,7 @@ class SQLAlchemyDatabase:
         *,
         threshold_for_copy: int = 100000,  # temporarily do not use copy
         returning_id: bool = False,
+        parallel: bool = False,
     ) -> pl.DataFrame | None:
         """
         Validate a Polars DataFrame against the model schema, then delegate
@@ -324,6 +327,13 @@ class SQLAlchemyDatabase:
             return None
 
         if len(data) >= threshold_for_copy:
+            if parallel:
+                return self.handle_insertion_multi_line_copy_parallel(
+                    model_class,
+                    data,
+                    mode,
+                    returning_id=returning_id,
+                )
             return self.handle_insertion_multi_line_copy(
                 model_class,
                 data,
@@ -619,8 +629,6 @@ class SQLAlchemyDatabase:
         try:
             # Connection tuning for bulk operations
             cursor.execute("SET synchronous_commit = OFF")
-            cursor.execute("SET wal_buffers = '16MB'")
-            cursor.execute("SET checkpoint_completion_target = 0.9")
 
             # Precompute ON CONFLICT clause based on mode
             conflict_cols = ", ".join(f'"{c}"' for c in pks)
@@ -644,7 +652,7 @@ class SQLAlchemyDatabase:
             cursor.execute(f'CREATE TEMP TABLE "{temp_table}" AS SELECT {non_id_col_list} FROM "{table}" WHERE FALSE')
 
             # Process each batch
-            for batch_idx, offset in enumerate(range(0, data.height, chunk_size)):
+            for offset in range(0, data.height, chunk_size):
                 batch = data.slice(offset, chunk_size)
                 if batch.height == 0:
                     continue
@@ -662,7 +670,7 @@ class SQLAlchemyDatabase:
                 insert_sql = (
                     f'INSERT INTO "{table}" ({non_id_col_list}) SELECT {non_id_col_list} FROM "{temp_table}" '
                     + on_conflict
-                )  # noqa: S608
+                )
                 cursor.execute(insert_sql)
 
                 # 4) Optionally retrieve IDs by joining via COPY + Polars
@@ -692,6 +700,110 @@ class SQLAlchemyDatabase:
             return all_dfs
         return None
 
+    def handle_insertion_multi_line_copy_parallel(
+        self,
+        model_class: DeclarativeMeta,
+        data: pl.DataFrame,
+        mode: t.Literal["RAISE", "IGNORE", "UPDATE"],
+        *,
+        chunk_size: int = 1_000_000,
+        num_workers: int = 4,
+        returning_id: bool = False,
+    ) -> pl.DataFrame | None:
+        """
+        Parallel bulk-load via PostgreSQL COPY using multiprocessing workers.
+
+        - Splits the data into chunks and distributes them across `num_workers` processes
+        - Each worker creates its own database connection and processes its assigned chunks
+        - For returning_id=True, collects results from all workers and combines them
+
+        """
+        print("using parallel COPY")
+        # Primary-key columns used for conflict detection
+        pks = self.get_primary_unique_constraint(model_class)
+        table = model_class.__table__.name
+        cols = data.columns
+
+        # Prepare batching
+        total_batches = (data.height + chunk_size - 1) // chunk_size
+        show_progress = total_batches > 1
+
+        logger.info(
+            "Starting parallel COPY insert of %d rows in %d chunks of size %d using %d workers",
+            data.height,
+            total_batches,
+            chunk_size,
+            num_workers,
+        )
+
+        # Split data into chunks for workers
+        chunks = []
+        for offset in range(0, data.height, chunk_size):
+            chunk = data.slice(offset, chunk_size)
+            if chunk.height > 0:
+                chunks.append((offset, chunk))
+
+        if not chunks:
+            return None
+
+        # Get connection parameters for workers to recreate engines
+        # Need to extract full connection info including password
+        engine_url = self.engine.url
+        connection_params = {
+            "host": engine_url.host,
+            "port": engine_url.port,
+            "database": engine_url.database,
+            "username": engine_url.username,
+            "password": engine_url.password,  # This will be None if not set
+        }
+
+        # If password is None, try to get the full URL with credentials
+        if connection_params["password"] is None:
+            # Try to get the URL with render_as_string to include password
+            connection_url = engine_url.render_as_string(hide_password=False)
+        else:
+            # Reconstruct connection URL with all parameters
+            connection_url = f"postgresql://{connection_params['username']}:{connection_params['password']}@{connection_params['host']}:{connection_params['port']}/{connection_params['database']}"
+
+        # Create worker function with connection string instead of engine
+        worker_func = partial(
+            _copy_worker_static,
+            connection_url=connection_url,
+            mode=mode,
+            pks=pks,
+            table=table,
+            cols=cols,
+            returning_id=returning_id,
+        )
+
+        # Process chunks in parallel
+        if show_progress:
+            progress_bar = tqdm(
+                total=data.height,
+                desc=f"Parallel COPY writing to table: {table}",
+                unit="rows",
+                dynamic_ncols=True,
+            )
+
+        all_id_maps = []
+
+        # Use multiprocessing Pool
+        with mp.Pool(processes=num_workers) as pool:
+            # Submit all chunks to workers
+            results = pool.map(worker_func, chunks)
+
+            # Collect results and update progress
+            for result in results:
+                batch_size, id_df = result
+                if show_progress:
+                    progress_bar.update(batch_size)
+                if returning_id and id_df is not None:
+                    all_id_maps.append(id_df)
+
+        if returning_id and all_id_maps:
+            return pl.concat(all_id_maps)
+        return None
+
     @staticmethod
     def get_column_attr_by_db_name(model_class: DeclarativeMeta, db_col_name: str) -> InstrumentedAttribute:
         """
@@ -758,3 +870,101 @@ class SQLAlchemyDatabase:
         traceback: TracebackType | None,
     ) -> None:
         self.close()
+
+
+def _copy_worker_static(
+    chunk_data: tuple[int, pl.DataFrame],
+    *,
+    connection_url: str,
+    mode: str,
+    pks: list[str],
+    table: str,
+    cols: list[str],
+    returning_id: bool,
+) -> tuple[int, pl.DataFrame | None]:
+    """
+    Static worker function for parallel COPY operations.
+    This function recreates its own database connection from the connection URL.
+
+    Returns:
+        tuple: (batch_size, id_dataframe_or_none)
+
+    """
+    offset, batch = chunk_data
+    batch_size = batch.height
+
+    # Create new engine and connection for this worker
+    worker_engine = create_engine(connection_url)
+    raw_conn = worker_engine.raw_connection()
+    cursor = raw_conn.cursor()
+
+    try:
+        # Connection tuning for bulk operations
+        cursor.execute("SET synchronous_commit = OFF")
+        cursor.execute("SET wal_buffers = '16MB'")
+        cursor.execute("SET checkpoint_completion_target = 0.9")
+
+        # Precompute ON CONFLICT clause based on mode
+        conflict_cols = ", ".join(f'"{c}"' for c in pks)
+        if mode == "RAISE":
+            on_conflict = ""
+        elif mode == "IGNORE":
+            on_conflict = f"ON CONFLICT ({conflict_cols}) DO NOTHING"
+        else:  # UPDATE
+            non_pk_cols = [c for c in cols if c not in pks]
+            if non_pk_cols:
+                set_clause = ", ".join(f'"{c}" = EXCLUDED."{c}"' for c in non_pk_cols)
+                on_conflict = f"ON CONFLICT ({conflict_cols}) DO UPDATE SET {set_clause}"
+            else:
+                on_conflict = f"ON CONFLICT ({conflict_cols}) DO NOTHING"
+
+        # Create temp table for this worker
+        temp_table = f"temp_{table}_worker_{offset}"
+        non_id_cols = [c for c in cols if c.upper() != "ID"]
+        non_id_col_list = ", ".join(f'"{c}"' for c in non_id_cols)
+
+        cursor.execute(
+            f'CREATE TEMP TABLE "{temp_table}" AS SELECT {non_id_col_list} FROM "{table}" WHERE FALSE'  # noqa: S608
+        )
+
+        # 1) COPY data into staging
+        buf = io.StringIO()
+        batch.write_csv(buf)
+        buf.seek(0)
+        cursor.copy_expert(f'COPY "{temp_table}" ({non_id_col_list}) FROM STDIN WITH CSV HEADER', buf)
+
+        # 2) INSERT FROM staging into real table
+        insert_sql = (
+            f'INSERT INTO "{table}" ({non_id_col_list}) SELECT {non_id_col_list} FROM "{temp_table}" '  # noqa: S608
+            + on_conflict
+        )
+        cursor.execute(insert_sql)
+
+        # 3) Optionally retrieve IDs
+        id_df = None
+        if returning_id:
+            buf_out = io.StringIO()
+            # Build SELECT clause with explicit aliases
+            select_list = ", ".join(['r."ID" AS "ID"', *[f't."{c}" AS "{c}"' for c in pks]])
+            copy_sql = f'COPY (SELECT {select_list} FROM "{table}" AS r JOIN "{temp_table}" AS t USING ({conflict_cols})) TO STDOUT WITH CSV HEADER'  # noqa: S608
+            cursor.copy_expert(copy_sql, buf_out)
+            buf_out.seek(0)
+            id_df = pl.read_csv(buf_out)
+
+        # 4) Clean up temp table
+        cursor.execute(f'DROP TABLE IF EXISTS "{temp_table}"')
+
+        # Commit this worker's transaction
+        raw_conn.commit()
+
+    except Exception:
+        raw_conn.rollback()
+        # Use print for worker logging since logger may not work across processes
+        print(f"Worker error for offset {offset}")
+        raise
+    else:
+        return (batch_size, id_df)
+    finally:
+        cursor.close()
+        raw_conn.close()
+        worker_engine.dispose()
